@@ -32,18 +32,23 @@ typedef struct _GIrModule GIrModule;
 struct _GIrModule
 { 
   gchar *name;
+  gchar *version;
   gchar *shared_library;
   GList *dependencies;
   GList *entries;
 };
 
 GIrModule *g_ir_module_new            (const gchar *name,
+				       const gchar *nsversion,
 				       const gchar *module_filename);
 void       g_ir_module_free           (GIrModule  *module);
 
 GTypelib * g_ir_module_build_typelib  (GIrModule  *module,
 				       GList       *modules);
 
+void _g_irnode_init_stats (void);
+void _g_irnode_dump_stats (void);
+
 G_END_DECLS
 
 #endif  /* __G_IR_MODULE_H__ */
