@@ -34,7 +34,7 @@ static gulong types_count = 0;
 static gulong unique_types_count = 0;
 
 void
-init_stats (void)
+_g_irnode_init_stats (void)
 {
   string_count = 0;
   unique_string_count = 0;
@@ -45,7 +45,7 @@ init_stats (void)
 }
 
 void
-dump_stats (void)
+_g_irnode_dump_stats (void)
 {
   g_message ("%lu strings (%lu before sharing), %lu bytes (%lu before sharing)",
 	     unique_string_count, string_count, unique_string_size, string_size);
@@ -339,6 +339,9 @@ g_ir_node_free (GIrNode *node)
 	GIrNodeStruct *struct_ = (GIrNodeStruct *)node;
 
 	g_free (node->name);
+	g_free (struct_->gtype_name);
+	g_free (struct_->gtype_init);
+
 	for (l = struct_->members; l; l = l->next)
 	  g_ir_node_free ((GIrNode *)l->data);
 	g_list_free (struct_->members);
@@ -706,6 +709,10 @@ g_ir_node_get_full_size_internal (GIrNode *parent,
 
 	size = 20;
 	size += ALIGN_VALUE (strlen (node->name) + 1, 4);
+	if (struct_->gtype_name)
+	  size += ALIGN_VALUE (strlen (struct_->gtype_name) + 1, 4);
+	if (struct_->gtype_init)
+	  size += ALIGN_VALUE (strlen (struct_->gtype_init) + 1, 4);
 	for (l = struct_->members; l; l = l->next)
 	  size += g_ir_node_get_full_size_internal (node, (GIrNode *)l->data);
       }
@@ -809,6 +816,10 @@ g_ir_node_get_full_size_internal (GIrNode *parent,
 
 	size = 28;
 	size += ALIGN_VALUE (strlen (node->name) + 1, 4);
+	if (union_->gtype_name)
+	  size += ALIGN_VALUE (strlen (union_->gtype_name) + 1, 4);
+	if (union_->gtype_init)
+	  size += ALIGN_VALUE (strlen (union_->gtype_init) + 1, 4);
 	for (l = union_->members; l; l = l->next)
 	  size += g_ir_node_get_full_size_internal (node, (GIrNode *)l->data);
 	for (l = union_->discriminators; l; l = l->next)
@@ -859,6 +870,25 @@ g_ir_node_can_have_member (GIrNode    *node)
     case G_IR_NODE_STRUCT:
     case G_IR_NODE_UNION:
       return TRUE;
+    /* list others individually rather than with default: so that compiler
+     * warns if new node types are added without adding them to the switch
+     */
+    case G_IR_NODE_INVALID:
+    case G_IR_NODE_FUNCTION:
+    case G_IR_NODE_CALLBACK:
+    case G_IR_NODE_ENUM:
+    case G_IR_NODE_FLAGS:
+    case G_IR_NODE_CONSTANT:
+    case G_IR_NODE_ERROR_DOMAIN:
+    case G_IR_NODE_PARAM:
+    case G_IR_NODE_TYPE:
+    case G_IR_NODE_PROPERTY:
+    case G_IR_NODE_SIGNAL:
+    case G_IR_NODE_VALUE:
+    case G_IR_NODE_VFUNC:
+    case G_IR_NODE_FIELD:
+    case G_IR_NODE_XREF:
+      return FALSE;
     };
   return FALSE;
 }
@@ -1075,8 +1105,7 @@ serialize_type (GIrModule    *module,
   
   if (node->tag < GI_TYPE_TAG_ARRAY)
     {
-      g_string_append_printf (str, "%s%s", 
-			      basic[node->tag], node->is_pointer ? "*" : "");
+      g_string_append_printf (str, "%s", basic[node->tag]);
     }
   else if (node->tag == GI_TYPE_TAG_ARRAY)
     {
@@ -1085,6 +1114,8 @@ serialize_type (GIrModule    *module,
 
       if (node->has_length)
 	g_string_append_printf (str, "length=%d", node->length);
+      else if (node->has_size)
+        g_string_append_printf (str, "fixed-size=%d", node->size);
       
       if (node->zero_terminated)
 	g_string_append_printf (str, "%szero-terminated=1", 
@@ -1106,7 +1137,7 @@ serialize_type (GIrModule    *module,
 	  name = node->interface;
 	}
 
-      g_string_append_printf (str, "%s%s", name, node->is_pointer ? "*" : "");
+      g_string_append_printf (str, "%s", name);
     }
   else if (node->tag == GI_TYPE_TAG_GLIST)
     {
@@ -1157,6 +1188,65 @@ serialize_type (GIrModule    *module,
     }
 }
 
+static void
+g_ir_node_build_members (GList         **members,
+			 GIrNodeTypeId   type,
+			 guint16        *count,
+			 GIrModule      *module,
+			 GList          *modules,
+			 GHashTable     *strings,
+			 GHashTable     *types,
+			 guchar         *data,
+			 guint32        *offset,
+			 guint32        *offset2)
+{
+  GList *l = *members;
+
+  while (l)
+    {
+      GIrNode *member = (GIrNode *)l->data;
+      GList *next = l->next;
+
+      if (member->type == type)
+	{
+	  (*count)++;
+	  g_ir_node_build_typelib (member, module, modules, strings,
+				   types, data, offset, offset2);
+	  *members = g_list_delete_link (*members, l);
+	}
+      l = next;
+    }
+}
+
+static void
+g_ir_node_check_unhandled_members (GList         **members,
+				   GIrNodeTypeId   container_type)
+{
+#if 0
+  if (*members)
+    {
+      GList *l;
+
+      for (l = *members; l; l = l->next)
+	{
+	  GIrNode *member = (GIrNode *)l->data;
+	  g_printerr ("Unhandled '%s' member '%s' type '%s'\n",
+		      g_ir_node_type_to_string (container_type),
+		      member->name,
+		      g_ir_node_type_to_string (member->type));
+	}
+
+      g_list_free (*members);
+      *members = NULL;
+
+      g_error ("Unhandled members. Aborting.");
+    }
+#else
+  g_list_free (*members);
+  *members = NULL;
+#endif
+}
+
 void
 g_ir_node_build_typelib (GIrNode    *node,
 			 GIrModule  *module,
@@ -1231,8 +1321,14 @@ g_ir_node_build_typelib (GIrNode    *node,
 		      array->tag = type->tag;
 		      array->zero_terminated = type->zero_terminated;
 		      array->has_length = type->has_length;
+                      array->has_size = type->has_size;
 		      array->reserved2 = 0;
-		      array->length = type->length;
+                      if (array->has_length)
+                        array->length = type->length;
+                      else if (array->has_size)
+                        array->size  = type->size;
+                      else
+                        array->length = -1;
 		      
 		      pos = *offset2 + 4;
 		      *offset2 += 8;
@@ -1391,7 +1487,7 @@ g_ir_node_build_typelib (GIrNode    *node,
 	blob->getter = function->is_getter;
 	blob->constructor = function->is_constructor;
 	blob->wraps_vfunc = function->wraps_vfunc;
-	blob->reserved = 0;
+	blob->throws = function->throws;
 	blob->index = 0;
 	blob->name = write_string (node->name, strings, data, offset2);
 	blob->symbol = write_string (function->symbol, strings, data, offset2);
@@ -1403,7 +1499,7 @@ g_ir_node_build_typelib (GIrNode    *node,
 				 module, modules, strings, types,
 				 data, &signature, offset2);
 
-	blob2->may_return_null = function->result->null_ok;
+	blob2->may_return_null = function->result->allow_none;
 	blob2->caller_owns_return_value = function->result->transfer;
 	blob2->caller_owns_return_container = function->result->shallow_transfer;
 	blob2->reserved = 0;
@@ -1447,7 +1543,7 @@ g_ir_node_build_typelib (GIrNode    *node,
 				 module, modules, strings, types,
 				 data, &signature, offset2);
 
-	blob2->may_return_null = function->result->null_ok;
+	blob2->may_return_null = function->result->allow_none;
 	blob2->caller_owns_return_value = function->result->transfer;
 	blob2->caller_owns_return_container = function->result->shallow_transfer;
 	blob2->reserved = 0;
@@ -1499,7 +1595,7 @@ g_ir_node_build_typelib (GIrNode    *node,
 				 module, modules, strings, types,
 				 data, &signature, offset2);
 
-	blob2->may_return_null = signal->result->null_ok;
+	blob2->may_return_null = signal->result->allow_none;
 	blob2->caller_owns_return_value = signal->result->transfer;
 	blob2->caller_owns_return_container = signal->result->shallow_transfer;
 	blob2->reserved = 0;
@@ -1546,7 +1642,7 @@ g_ir_node_build_typelib (GIrNode    *node,
 				 module, modules, strings, types,
 				 data, &signature, offset2);
 
-	blob2->may_return_null = vfunc->result->null_ok;
+	blob2->may_return_null = vfunc->result->allow_none;
 	blob2->caller_owns_return_value = vfunc->result->transfer;
 	blob2->caller_owns_return_container = vfunc->result->shallow_transfer;
 	blob2->reserved = 0;
@@ -1575,7 +1671,7 @@ g_ir_node_build_typelib (GIrNode    *node,
  	blob->in = param->in;
  	blob->out = param->out;
  	blob->dipper = param->dipper;
-	blob->null_ok = param->null_ok;
+	blob->allow_none = param->allow_none;
 	blob->optional = param->optional;
 	blob->transfer_ownership = param->transfer;
 	blob->transfer_container_ownership = param->shallow_transfer;
@@ -1591,42 +1687,43 @@ g_ir_node_build_typelib (GIrNode    *node,
       {
 	StructBlob *blob = (StructBlob *)&data[*offset];
 	GIrNodeStruct *struct_ = (GIrNodeStruct *)node;
+	GList *members;
 	
 	blob->blob_type = BLOB_TYPE_STRUCT;
 	blob->deprecated = struct_->deprecated;
-	blob->unregistered = TRUE;
 	blob->reserved = 0;
 	blob->name = write_string (node->name, strings, data, offset2);
-	blob->gtype_name = 0;
-	blob->gtype_init = 0;
+	if (struct_->gtype_name)
+	  {
+	    blob->unregistered = FALSE;
+	    blob->gtype_name = write_string (struct_->gtype_name, strings, data, offset2);
+	    blob->gtype_init = write_string (struct_->gtype_init, strings, data, offset2);
+	  }
+	else
+	  {
+	    blob->unregistered = TRUE;
+	    blob->gtype_name = 0;
+	    blob->gtype_init = 0;
+	  }
 
 	blob->n_fields = 0;
 	blob->n_methods = 0;
 
 	*offset += 20; 
-	for (l = struct_->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
 
-	    if (member->type == G_IR_NODE_FIELD)
-	      {
-		blob->n_fields++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	members = g_list_copy (struct_->members);
 
-	for (l = struct_->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
-	    
-	    if (member->type == G_IR_NODE_FUNCTION)
-	      {
-		blob->n_methods++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	g_ir_node_build_members (&members, G_IR_NODE_FIELD, &blob->n_fields,
+				 module, modules, strings,
+				 types, data, offset, offset2);
+
+	g_ir_node_build_members (&members, G_IR_NODE_FUNCTION, &blob->n_methods,
+				 module, modules, strings,
+				 types, data, offset, offset2);
+
+	g_ir_node_check_unhandled_members (&members, node->type);
+
+	g_assert (members == NULL);
       }
       break;
 
@@ -1634,6 +1731,7 @@ g_ir_node_build_typelib (GIrNode    *node,
       {
 	StructBlob *blob = (StructBlob *)&data[*offset];
 	GIrNodeBoxed *boxed = (GIrNodeBoxed *)node;
+	GList *members;
 
 	blob->blob_type = BLOB_TYPE_BOXED;
 	blob->deprecated = boxed->deprecated;
@@ -1647,29 +1745,20 @@ g_ir_node_build_typelib (GIrNode    *node,
 	blob->n_methods = 0;
 
 	*offset += 20; 
-	for (l = boxed->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
 
-	    if (member->type == G_IR_NODE_FIELD)
-	      {
-		blob->n_fields++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	members = g_list_copy (boxed->members);
 
-	for (l = boxed->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
+	g_ir_node_build_members (&members, G_IR_NODE_FIELD, &blob->n_fields,
+				 module, modules, strings,
+				 types, data, offset, offset2);
 
-	    if (member->type == G_IR_NODE_FUNCTION)
-	      {
-		blob->n_methods++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	g_ir_node_build_members (&members, G_IR_NODE_FUNCTION, &blob->n_methods,
+				 module, modules, strings,
+				 types, data, offset, offset2);
+
+	g_ir_node_check_unhandled_members (&members, node->type);
+
+	g_assert (members == NULL);
       }
       break;
 
@@ -1677,6 +1766,7 @@ g_ir_node_build_typelib (GIrNode    *node,
       {
 	UnionBlob *blob = (UnionBlob *)&data[*offset];
 	GIrNodeUnion *union_ = (GIrNodeUnion *)node;
+	GList *members;
 
 	blob->blob_type = BLOB_TYPE_UNION;
 	blob->deprecated = union_->deprecated;
@@ -1715,30 +1805,19 @@ g_ir_node_build_typelib (GIrNode    *node,
 	    blob->discriminator_type.offset = 0;
 	  }
 	
-	
-	for (l = union_->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
+	members = g_list_copy (union_->members);
 
-	    if (member->type == G_IR_NODE_FIELD)
-	      {
-		blob->n_fields++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	g_ir_node_build_members (&members, G_IR_NODE_FIELD, &blob->n_fields,
+				 module, modules, strings,
+				 types, data, offset, offset2);
 
-	for (l = union_->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
+	g_ir_node_build_members (&members, G_IR_NODE_FUNCTION, &blob->n_functions,
+				 module, modules, strings,
+				 types, data, offset, offset2);
 
-	    if (member->type == G_IR_NODE_FUNCTION)
-	      {
-		blob->n_functions++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	g_ir_node_check_unhandled_members (&members, node->type);
+
+	g_assert (members == NULL);
 
 	if (union_->discriminator_type)
 	  {
@@ -1800,8 +1879,10 @@ g_ir_node_build_typelib (GIrNode    *node,
       {
 	ObjectBlob *blob = (ObjectBlob *)&data[*offset];
 	GIrNodeInterface *object = (GIrNodeInterface *)node;
+	GList *members;
 
 	blob->blob_type = BLOB_TYPE_OBJECT;
+	blob->abstract = object->abstract;
 	blob->deprecated = object->deprecated;
 	blob->reserved = 0;
 	blob->name = write_string (node->name, strings, data, offset2);
@@ -1828,83 +1909,41 @@ g_ir_node_build_typelib (GIrNode    *node,
 	    *offset += 2;
 	  }
 	
-	*offset = ALIGN_VALUE (*offset, 4);
-	for (l = object->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
-
-	    if (member->type == G_IR_NODE_FIELD)
-	      {
-		blob->n_fields++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	members = g_list_copy (object->members);
 
 	*offset = ALIGN_VALUE (*offset, 4);
-	for (l = object->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
-
-	    if (member->type == G_IR_NODE_PROPERTY)
-	      {
-		blob->n_properties++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	g_ir_node_build_members (&members, G_IR_NODE_FIELD, &blob->n_fields,
+				 module, modules, strings,
+				 types, data, offset, offset2);
 
 	*offset = ALIGN_VALUE (*offset, 4);
-	for (l = object->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
-
-	    if (member->type == G_IR_NODE_FUNCTION)
-	      {
-		blob->n_methods++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	g_ir_node_build_members (&members, G_IR_NODE_PROPERTY, &blob->n_properties,
+				 module, modules, strings,
+				 types, data, offset, offset2);
 
 	*offset = ALIGN_VALUE (*offset, 4);
-	for (l = object->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
-
-	    if (member->type == G_IR_NODE_SIGNAL)
-	      {
-		blob->n_signals++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	g_ir_node_build_members (&members, G_IR_NODE_FUNCTION, &blob->n_methods,
+				 module, modules, strings,
+				 types, data, offset, offset2);
 
 	*offset = ALIGN_VALUE (*offset, 4);
-	for (l = object->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
-
-	    if (member->type == G_IR_NODE_VFUNC)
-	      {
-		blob->n_vfuncs++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	g_ir_node_build_members (&members, G_IR_NODE_SIGNAL, &blob->n_signals,
+				 module, modules, strings,
+				 types, data, offset, offset2);
 
 	*offset = ALIGN_VALUE (*offset, 4);
-	for (l = object->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
+	g_ir_node_build_members (&members, G_IR_NODE_VFUNC, &blob->n_vfuncs,
+				 module, modules, strings,
+				 types, data, offset, offset2);
 
-	    if (member->type == G_IR_NODE_CONSTANT)
-	      {
-		blob->n_constants++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	*offset = ALIGN_VALUE (*offset, 4);
+	g_ir_node_build_members (&members, G_IR_NODE_CONSTANT, &blob->n_constants,
+				 module, modules, strings,
+				 types, data, offset, offset2);
+
+	g_ir_node_check_unhandled_members (&members, node->type);
+
+	g_assert (members == NULL);
       }
       break;
 
@@ -1912,6 +1951,7 @@ g_ir_node_build_typelib (GIrNode    *node,
       {
 	InterfaceBlob *blob = (InterfaceBlob *)&data[*offset];
 	GIrNodeInterface *iface = (GIrNodeInterface *)node;
+	GList *members;
 
 	blob->blob_type = BLOB_TYPE_INTERFACE;
 	blob->deprecated = iface->deprecated;
@@ -1934,70 +1974,36 @@ g_ir_node_build_typelib (GIrNode    *node,
 	    *offset += 2;
 	  }
 	
-	*offset = ALIGN_VALUE (*offset, 4);
-	for (l = iface->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
-
-	    if (member->type == G_IR_NODE_PROPERTY)
-	      {
-		blob->n_properties++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	members = g_list_copy (iface->members);
 
 	*offset = ALIGN_VALUE (*offset, 4);
-	for (l = iface->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
-
-	    if (member->type == G_IR_NODE_FUNCTION)
-	      {
-		blob->n_methods++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	g_ir_node_build_members (&members, G_IR_NODE_PROPERTY, &blob->n_properties,
+				 module, modules, strings,
+				 types, data, offset, offset2);
 
 	*offset = ALIGN_VALUE (*offset, 4);
-	for (l = iface->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
-
-	    if (member->type == G_IR_NODE_SIGNAL)
-	      {
-		blob->n_signals++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	g_ir_node_build_members (&members, G_IR_NODE_FUNCTION, &blob->n_methods,
+				 module, modules, strings,
+				 types, data, offset, offset2);
 
 	*offset = ALIGN_VALUE (*offset, 4);
-	for (l = iface->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
-
-	    if (member->type == G_IR_NODE_VFUNC)
-	      {
-		blob->n_vfuncs++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	g_ir_node_build_members (&members, G_IR_NODE_SIGNAL, &blob->n_signals,
+				 module, modules, strings,
+				 types, data, offset, offset2);
 
 	*offset = ALIGN_VALUE (*offset, 4);
-	for (l = iface->members; l; l = l->next)
-	  {
-	    GIrNode *member = (GIrNode *)l->data;
+	g_ir_node_build_members (&members, G_IR_NODE_VFUNC, &blob->n_vfuncs,
+				 module, modules, strings,
+				 types, data, offset, offset2);
 
-	    if (member->type == G_IR_NODE_CONSTANT)
-	      {
-		blob->n_constants++;
-		g_ir_node_build_typelib (member, module, modules, strings, 
-					 types, data, offset, offset2);
-	      }
-	  }
+	*offset = ALIGN_VALUE (*offset, 4);
+	g_ir_node_build_members (&members, G_IR_NODE_CONSTANT, &blob->n_constants,
+				 module, modules, strings,
+				 types, data, offset, offset2);
+
+	g_ir_node_check_unhandled_members (&members, node->type);
+
+	g_assert (members == NULL);
       }
       break;
 
@@ -2098,6 +2104,7 @@ g_ir_node_build_typelib (GIrNode    *node,
 	    *(glong*)&data[blob->offset] = (glong) parse_int_value (constant->value);
 	    break;
 	  case GI_TYPE_TAG_SIZE: /* FIXME */
+	  case GI_TYPE_TAG_TIME_T: 
 	  case GI_TYPE_TAG_ULONG:
 	    blob->size = sizeof (gulong);
 	    *(gulong*)&data[blob->offset] = (gulong) parse_uint_value (constant->value);
