@@ -85,12 +85,14 @@ struct _GIrNodeFunction
   GIrNode node;
 
   gboolean deprecated;
+  gboolean is_varargs; /* Not in typelib yet */ 
 
   gboolean is_method;
   gboolean is_setter;
   gboolean is_getter;
   gboolean is_constructor;
   gboolean wraps_vfunc;
+  gboolean throws;
 
   gchar *symbol;
 
@@ -117,6 +119,8 @@ struct _GIrNodeType
   gboolean zero_terminated;
   gboolean has_length;
   gint length;
+  gboolean has_size;
+  gint size;
   
   GIrNodeType *parameter_type1;
   GIrNodeType *parameter_type2;  
@@ -134,7 +138,7 @@ struct _GIrNodeParam
   gboolean dipper;
   gboolean optional;
   gboolean retval;
-  gboolean null_ok;
+  gboolean allow_none;
   gboolean transfer;
   gboolean shallow_transfer;
   
@@ -183,6 +187,7 @@ struct _GIrNodeVFunc
 {
   GIrNode node;
 
+  gboolean is_varargs; /* Not in typelib yet */ 
   gboolean must_chain_up;
   gboolean must_be_implemented;
   gboolean must_not_be_implemented;
@@ -210,6 +215,7 @@ struct _GIrNodeInterface
 {
   GIrNode node;
 
+  gboolean abstract;
   gboolean deprecated;
 
   gchar *gtype_name;
@@ -272,6 +278,9 @@ struct _GIrNodeStruct
   GIrNode node;
 
   gboolean deprecated;
+
+  gchar *gtype_name;
+  gchar *gtype_init;
   
   GList *members;
 };
