@@ -53,6 +53,8 @@ UtilityObject*        foo_object_external_type     (FooObject *object);
 
 void                  foo_object_various           (FooObject *object, void *data, GType some_type);
 
+void                  foo_object_take_all          (FooObject *object, int x, ...);
+
 /* A random typedef */
 typedef GSList FooList;
 
@@ -62,6 +64,14 @@ typedef gpointer FooObjectCookie;
 
 FooObjectCookie       foo_object_new_cookie        (FooObject *object, const char *target);
 
+void                  foo_object_is_it_time_yet    (FooObject *object, time_t time);
+
+const char *          foo_object_get_name          (FooObject *object);
+
+char *                foo_object_dup_name          (FooObject *object);
+
+void                  foo_object_handle_glyph      (FooObject *object, UtilityGlyph glyph);
+
 struct _FooSubobject
 {
   FooObject parent_instance;
@@ -100,16 +110,29 @@ typedef enum
   FOO_ENUM_UN = 1,
   FOO_ENUM_DEUX = 2,
   FOO_ENUM_TROIS = 3,
+  FOO_ENUM_NEUF = 9
 } FooEnumNoType;
 
 GType foo_flags_type_get_type (void);
 
+typedef enum
+{
+  FOO_ENUM_FULLNAME_ONE = 1,
+  FOO_ENUM_FULLNAME_TWO,
+  FOO_ENUM_FULLNAME_THREE
+} FooEnumFullname;
+
 typedef struct _FooBoxed FooBoxed;
 
 GType                 foo_boxed_get_type       (void) G_GNUC_CONST;
 FooBoxed*             foo_boxed_new            (void);
 void                  foo_boxed_method         (FooBoxed* boxed);
 
+/* This one tests capitalization handling with respect to DBus */
+typedef struct _FooDBusData FooDBusData;
+GType                 foo_dbus_data_get_type       (void) G_GNUC_CONST;
+void                  foo_dbus_data_method         (FooDBusData* dbusdata);
+
 /* FIXME: Scanner does not support this yet
 const char *FOO_CONSTANT_STR = "foo-constant-str";
 const int FOO_CONSTANT_INT = 10;
@@ -151,6 +174,9 @@ foo_method_external_references (UtilityObject *object,
 
 void foo_rectangle_add(FooRectangle *r1, const FooRectangle *r2);
 
+FooRectangle * foo_rectangle_new (int x, int y, int width, int height);
+
+
 typedef struct _FooEventAny FooEventAny;
 typedef struct _FooEventExpose FooEventExpose;
 
@@ -176,4 +202,47 @@ union _FooEvent
 
 typedef void FooXEvent;
 
+/* And now some boxed variants */
+
+typedef struct _FooBRect          FooBRect;
+
+struct _FooBRect
+{
+  double x;
+  double y;
+};
+
+GType foo_brect_get_type (void);
+
+FooBRect *foo_brect_new (double x, double y);
+
+void foo_brect_add (FooBRect *b1, FooBRect *b2);
+
+typedef union _FooBUnion FooBUnion;
+
+union _FooBUnion
+{
+  int type;
+  double v;
+  FooBRect *rect;
+};
+
+typedef union _FooUnion
+{
+  int foo;
+} FooUnion;
+
+FooBUnion *foo_bunion_new (void);
+
+GType foo_bunion_get_type (void);
+
+int foo_bunion_get_contained_type (FooBUnion *bunion);
+
+void foo_test_unsigned_qualifier (unsigned int unsigned_param);
+void foo_test_unsigned_type (unsigned unsigned_param);
+
+void foo_test_string_array (char **array);
+
+void foo_test_string_array_with_g (gchar **array);
+
 #endif /* __FOO_OBJECT_H__ */
